% eegthresh() - classical trial rejection rejection using a threshold on the raw signal
%
% Usage:
%   >> [Iin, Iout, newsignal] = ...
%                 eegthresh( signal, frames, electrodes, threshold, timerange, timeselect);
%
% Inputs:
%   signal     - signal 2D, channels x (frames*sweeps) or 3D channels x frames x sweeps
%   frames     - number of points per epoch
%   electrodes - [e1 e2 ...] electrodes (number) to take into consideration for rejection
%   threshold  - [minthresh maxthresh] threshold limit in mV
%   timerange  - [mintime maxtime] timerange limit in second
%   timeselect  - [mintime maxtime] timeselect limit in second
%
% Outputs:
%   Iin        - Indexes of sweeps not rejected
%   Iout       - Indexes of sweeps rejected
%   newsignal  - signal after sweeps rejection (same number of dimension as signal)

% arno@salk.edu, Arnaud Delorme, CNL / Salk Institute, 2001

% This program is free software; you can redistribute it and/or
% modify it.  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

function [Iin, Iout, newsignal] = eegthresh( signal, pnts, electrodes, threshold, timerange, timeselect);

if nargin < 6
	help eegthresh;
	return;
end;

% transform the time range
%-------------------------
framelowlimit  = floor((timeselect(1)-timerange(1))/(timerange(2)-timerange(1))*(pnts-1))+1;
framehighlimit = floor((timeselect(2)-timerange(1))/(timerange(2)-timerange(1))*(pnts-1))+1;

% find the maximum for each trial
%--------------------------------
sweeps = size(signal(1,:),2)/pnts;
sigtmp = reshape(signal(electrodes,:), size(electrodes(:),1), pnts, sweeps);
sigtmp = sigtmp(:,framelowlimit:framehighlimit,:);
sigmax = squeeze(max(sigtmp,[],2));
sigmax = squeeze(max(sigmax,[],1));
sigmin = squeeze(min(sigtmp,[],2));
sigmin = squeeze(min(sigmin,[],1));

% reject the selected trials
%---------------------------
Iout = union( find( sigmin < threshold(1) ),  find( sigmax > threshold(2) ) );
Iin  = setdiff([1:sweeps], Iout);

% reject the selected trials
%---------------------------
newsignal = reshape(signal, size(signal,1), pnts, sweeps);
if ~isempty(Iin)
	newsignal = newsignal(:,:,Iin);
	if ndims(signal) == 2
		newsignal = newsignal(:,:);
	end;
else
	newsignal = [];
end;
return;