% rejectrials - reject bad eeg trials
%
% Usage:
%   >> [I, newsignal, n, threshold, E] = rejectrials( signal, pnts, plot, threshold, E);
%
% Inputs:
%   signal     - 1 dimentionnal or 2 dimentionnal (if 2 dimentionnal
%                the first dimention must be composed of independant
%                components
%   pnts       - number of points per trials
%
% Optional 
%   plot       - [0|1] plot the normalized entropy of original signal (blue) 
%                and the limits (red) (default:0)            
%   threshold  - percentage error threshold (default 1-0.25/nb_trials)
%   E          - precomputed normalized entropy distribution
%
% Outputs:
%   I          - indexes to be rejected
%   newsignal  - signal without the rejected trials
%   n          - number of trials rejected
%   threshold  - percentage error threshold 
%   E          - entropy distribution
%
% Algorithm:
%   Compute the entropy of the signal (need the function trialentropy)
%   normalise the entropy and reject trials based on an uniform distribution
%   of the data (the entropy distribution of a random variable is
%   gaussian)

% arno@salk.edu, Arnaud Delorme, CNL / Salk Institute, 2001

% This program is free software; you can redistribute it and/or
% modify it.  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

function [I, newsignal, n, threshold, E] = rejectrials( signal, pnts, pp, threshold, E);

if nargin < 2
	help rejectrials;
	return;
end;	
if nargin < 4
	threshold = 1-0.25*pnts/size(signal(1,:),2);
else if threshold == 1 
		threshold = 1-0.25*pnts/size(signal(1,:),2);
	 end;
end;
if nargin < 3
	pp = 0;
end;

% compute the entropy of the signal
% --------------------------------
if ~exist('E')
	[E EM] = trialentropy( signal, pnts );

	% normalize E
	% -----------	
	E = (E-mean(E))/std(E);
end;

I = find( abs(E) < erfinv( threshold ) );

% compute new signal
% ------------------
if nargout > 1
	signal = reshape( signal, size(signal,1), pnts, size(signal(1,:),2)/pnts);
	n = size(signal,3) - size(I(:),1);
	newsignal = signal( :, :, I);
	newsignal = newsignal(:,:);
end;	

if pp
	% plot the sorted curve
	% ---------------------
	set(gcf, 'name', 'Rejectrials');
	MARGIN = 0.1;	
	ybase = 0.0;
	subheight = 0.3;
	colormap(jet(256));
	pos = get(gca,'position'); % plot relative to current axes
	q = [pos(1) pos(2) 0 0];
	s = [pos(3) pos(4) pos(3) pos(4)]./100;
	axis('off');
	h(6) = axes('Units','Normalized', 'Position',[0 70 100 30].*s+q);
	plot([1:size(E(:),1)], sort(E)); 
	hold on;
	yl = get(gca, 'ylim');
	%Itmp = find( E > erfinv( threshold ) );
	%Xtmp = size(Itmp(:), 1);
	%if Xtmp ~= 0
	%	plot([Xtmp Xtmp], yl, 'red');
	%end;
	Itmp = find( E < -erfinv( threshold ) );
	Xtmp = size(Itmp(:), 1);
	if Xtmp ~= 0
		plot([Xtmp Xtmp], yl, 'red');
	end;
	title(sprintf('Entropy of trials (sorted), red line = error threshold(%f)',  1-threshold));
	%fprintf('To scale the initial part, type set(gca, ''xlim'', [0 %d])\n', round( size(E(:),1) / 50));

	% plot the datas
	% --------------
	hold on;
	h(7) = axes('Units','Normalized', 'Position',[0 0 42 60].*s+q);
	signal = signal(:,:);
	erpimage( mean(signal,1), ones(1,size(signal,2)/pnts), [0:1/(pnts-1):1], '', 3, 1, 'nosort', 'noplot');
	title('Currentset all chans'); xlabel(''); ylabel('');
	ss = get(gca, 'YLim');
	hold on;
	h(8) = axes('Units','Normalized', 'Position',[45 0 10 60].*s+q);
	EM = zeros(10, size(E(:),1));
	EM(:,Itmp) = ones(10, size(Itmp(:),1));
	if any(EM(:) ~= 0)
		erpimage( EM(:)', ones(1,size(E(:),1)), [0:1/(10-1):1], '', 1, 0, 'nosort', 'noplot');
	end;
	title('Suppressed trials'); xlabel(''); ylabel('');
	set(gca, 'yticklabel', []);
	set(gca, 'yticklabel', []);
	hold on;
	h(9) = axes('Units','Normalized', 'Position',[58 0 42 60].*s+q);
	plot(E);
	hold on;
	position = max(E(Itmp));
	plot(get(gca,'xlim'), [position position], 'red');
	set(gca, 'view', [90 90]);
	set(gca, 'xdir', 'reverse');
	set(gca, 'XLim', ss);
	set(gca, 'Xticklabel', []);
	title('Trial''s entropy');
end;
return;

	% approximate linearly the entropy
	% --------------------------------
	EE = sort(E);
	EE2 = sort(E(I));
	EE = (EE-mean(EE(:)))/std(EE(:));
	slope = - ( EE( round( size(E(:),1) / 100) ) - EE( round( 80 * size(E(:),1) / 100) ) ) / round( 79 * size(E(:),1)/100);
	intersect = EE( round( 5 * size(E(:),1) / 10) ) - slope * round( 5 * size(E(:),1) / 10);
	xx = 1:size(EE,1);
	yy = intersect + slope*xx;
	figure
	plot(EE);
	hold on;
	plot(yy, 'red');  

	% compare with the error function
	% -------------------------------
	otherdat = linspace(-1, 1, size( EE(:), 1));
	plot( erfinv(otherdat), 'green' );

